'use strict';

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _generate = require('../js/generate.js');

var _generate2 = _interopRequireDefault(_generate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var WHITELIST = ['additionalItems.json', 'additionalProperties.json', 'allOf.json', 'anyOf.json', 'default.json',
// 'definitions.json',
'dependencies.json', 'enum.json', 'items.json', 'maxItems.json', 'maxLength.json', 'maxProperties.json', 'maximum.json', 'minItems.json', 'minLength.json', 'minProperties.json', 'minimum.json', 'multipleOf.json', 'not.json', 'oneOf.json', 'optional.json', 'pattern.json', 'patternProperties.json', 'properties.json', 'ref.json',
// 'refRemote.json',
'required.json', 'type.json', 'uniqueItems.json'];
var SKIPPED_TESTS = {
  'items.json': {
    'an array of schemas for items': {
      'JavaScript pseudo-array is valid': false
    }
  },
  'maxLength.json': {
    'maxLength validation': {
      'two supplementary Unicode code points is long enough': false
    }
  },
  'minLength.json': {
    'minLength validation': {
      'one supplementary Unicode code point is not long enough': false
    }
  },
  'ref.json': {
    'remote ref, containing refs itself': {
      'remote ref invalid': false
    },
    'Recursive references between schemas': {
      'invalid tree': false
    }
  }
};

describe('JSON Schema test suite', function () {
  var testsPath = _path2.default.join(__dirname, '../../', 'JSON-Schema-Test-Suite/tests/draft4/');
  var testFiles = _fs2.default.readdirSync(testsPath).filter(function (fileName) {
    return WHITELIST.indexOf(fileName) >= 0;
  });
  testFiles.forEach(function (fileName) {
    describe(fileName, function () {
      // $FlowFixMe Dynamic require for DRY
      var groups = require(_path2.default.join(testsPath, fileName));
      groups.forEach(function (group) {
        var description = group.description,
            schema = group.schema,
            tests = group.tests;

        var code = (0, _generate2.default)(schema);
        it('matches snapshot', function () {
          expect(code).toMatchSnapshot();
        });
        var validator = eval(code).root; // eslint-disable-line no-eval
        tests.forEach(function (test) {
          describe(description, function () {
            var testDescription = test.description,
                data = test.data,
                valid = test.valid;

            if (_lodash2.default.get(SKIPPED_TESTS, [fileName, description, testDescription], true)) {
              it(testDescription, function () {
                expect(validator(data) === 0).toBe(valid);
              });
            }
          });
        });
      });
    });
  });
});